@@ -1,8 +1,8 @@
 # -*- perl -*-
 
-package Bundle::Msql;
+package Bundle::Mysql;
 
-$VERSION = '1.19_11';
+$VERSION = '1.1817;
 
 1;
 
