@@ -16,7 +16,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '2.01_11';
+$VERSION = '2.006';
 
 bootstrap DBD::mSQL $VERSION;
 
@@ -42,62 +42,10 @@ sub driver{
     $drh;
 }
 
-sub _OdbcParse($$$) {
-    my($class, $dsn, $hash, $args) = @_;
-    my($var, $val);
-    if (!defined($dsn)) {
-	return;
-    }
-    while (length($dsn)) {
-	if ($dsn =~ /([^:;]*)[:;](.*)/) {
-	    $val = $1;
-	    $dsn = $2;
-	} else {
-	    $val = $dsn;
-	    $dsn = '';
-	}
-	if ($val =~ /([^=]*)=(.*)/) {
-	    $var = $1;
-	    $val = $2;
-	    if ($var eq 'hostname') {
-		$hash->{'host'} = $val;
-	    } elsif ($var eq 'db'  ||  $var eq 'dbname') {
-		$hash->{'database'} = $val;
-	    } else {
-		$hash->{$var} = $val;
-	    }
-	} else {
-	    foreach $var (@$args) {
-		if (!defined($hash->{$var})) {
-		    $hash->{$var} = $val;
-		    last;
-		}
-	    }
-	}
-    }
-}
-
-sub _OdbcParseHost ($$) {
-    my($class, $dsn) = @_;
-    my($hash) = {};
-    $class->_OdbcParse($dsn, $hash, ['host', 'port']);
-    ($hash->{'host'}, $hash->{'port'});
-}
-
-sub AUTOLOAD {
-    my ($meth) = $DBD::mSQL::AUTOLOAD;
-    my ($smeth) = $meth;
-    $smeth =~ s/(.*)\:\://;
-
-    my $val = constant($smeth, @_ ? $_[0] : 0);
-    if ($! == 0) { eval "sub $meth { $val }"; return $val; }
-
-    Carp::croak "$meth: Not defined";
-}
-
 1;
 
 
+
 package DBD::mSQL::dr; # ====== DRIVER ======
 use strict;
 
@@ -111,22 +59,14 @@ sub connect {
     $password ||= '';
 
     # create a 'blank' dbh
-    my($this, $privateAttrHash);
-    $privateAttrHash = {
-	'dsn' => $dsn,
-	'user' => $username,
-	'password' => $password
-    };
+    my $this;
 
-    DBD::mSQL->_OdbcParse($dsn, $privateAttrHash,
-				  ['database', 'host', 'port']);
-
-    if (!defined($this = DBI::_new_dbh($drh, {}, $privateAttrHash))) {
+    if (!defined($this = DBI::_new_dbh($drh, { 'dsn' => $dsn,
+					       'user' => $username
+                                             }))) {
 	return undef;
     }
 
-    $this->{'private_DBD_mSQL_attr'} = $privateAttrHash;
-
     # Call msqlConnect func in mSQL.xs file
     # and populate internal handle data.
     DBD::mSQL::db::_login($this, $dsn, $username, $password)
@@ -144,42 +84,6 @@ sub data_sources {
     @dsn;
 }
 
-sub admin {
-    my($drh) = shift;
-    my($command) = shift;
-    my($dbname) = ($command eq 'createdb'  ||  $command eq 'dropdb') ?
-	shift : '';
-    my($host, $port) = DBD::mSQL->_OdbcParseHost(shift(@_) || '');
-    my($user) = shift || '';
-    my($password) = shift || '';
-
-    $drh->func(undef, $command,
-	       $dbname || '',
-	       $host || '',
-	       $port || '',
-	       $user, $password, '_admin_internal');
-}
-
-sub _CreateDB {
-    my($drh) = shift;
-    my($host) = (@_ > 1) ? shift : undef;
-    my($dbname) = shift;
-    if (!$DBD::mSQL::QUIET) {
-	warn "'_CreateDB' is deprecated, use 'admin' instead";
-    }
-    $drh->func('createdb', $dbname, $host, 'admin');
-}
-
-sub _DropDB {
-    my($drh) = shift;
-    my($host) = (@_ > 1) ? shift : undef;
-    my($dbname) = shift;
-    if (!$DBD::mSQL::QUIET) {
-	warn "'DropDB' is deprecated, use 'admin' instead";
-    }
-    $drh->func('dropdb', $dbname, $host, 'admin');
-}
-
 
 package DBD::mSQL::db; # ====== DATABASE ======
 use strict;
@@ -240,46 +144,34 @@ sub ANSI2db {
     return $DBD::mSQL::db::ANSI2db{"$type"};
 }
 
+sub quote {
+    my ($self, $str) = @_;
+    if (!defined($str)) { return 'NULL'; }
+    $str =~ s/([\0\\\'])/\\$1/g;
+    "'$str'";
+}
+
+# Just a stub for backward compatibility; use is deprecated
 sub _ListFields($$) {
-    my($self, $table) = @_;
-    if (!$DBD::mSQL::QUIET) {
-	warn "'_ListFields' is deprecated, use the SQL query 'LISTFIELDS \$table' instead.";
+    my ($self, $table) = @_;
+    my ($sth);
+    if (!($sth = $self->prepare("LISTFIELDS $table"))) {
+	return undef;
     }
-    my($sth) = $self->prepare("LISTFIELDS $table");
-    if ($sth  &&  !$sth->execute()) {
-	undef $sth;
+    if (!$sth->execute) {
+	return undef;
     }
     $sth;
 }
 
-sub admin {
-    my($dbh) = shift;
-    my($command) = shift;
-    my($dbname) = ($command eq 'createdb'  ||  $command eq 'dropdb') ?
-	shift : '';
-    $dbh->{'Driver'}->func($dbh, $command, $dbname, '', '', '',
-			   '_admin_internal');
-}
-
-
 package DBD::mSQL::st; # ====== STATEMENT ======
 use strict;
 
 # Just a stub for backward compatibility; use is deprecated
 sub _ListSelectedFields ($) {
-    if (!$DBD::mSQL::QUIET) {
-	warn "_ListSelectedFields is deprecated and superfluos";
-    }
     shift;
 }
 
-sub _NumRows ($) {
-    if (!$DBD::mSQL::QUIET) {
-	warn "_NumRows is deprecated, use \$sth->rows instead.";
-    }
-    shift->rows;
-}
-
 1;
 
 
@@ -302,48 +194,28 @@ Interface (DBI)
     $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
 			$user, $password);
 
-    @databases = DBD::mysql::dr->func($host, $port, '_ListDBs');
+    @databases = DBD::mysql::dr->func( $hostname, '_ListDBs' );
     @tables = $dbh->func( '_ListTables' );
 
-    $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
-       or
     $sth = $dbh->prepare("LISTFIELDS $table");
-       or
-    $sth = $dbh->prepare("LISTINDEX $table $index");
+    $sth->execute;
+    $sth->finish;
+
+    $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
     $sth->execute;
     $numRows = $sth->rows;
     $numFields = $sth->{'NUM_OF_FIELDS'};
     $sth->finish;
 
-    $rc = $drh->func('createdb', $database, $host, $user, $password, 'admin');
-    $rc = $drh->func('dropdb', $database, $host, $user, $password, 'admin');
-    $rc = $drh->func('shutdown', $host, $user, $password, 'admin');
-    $rc = $drh->func('reload', $host, $user, $password, 'admin');
-
-    $rc = $dbh->func('createdb', $database, 'admin');
-    $rc = $dbh->func('dropdb', $database, 'admin');
-    $rc = $dbh->func('shutdown', 'admin');
-    $rc = $dbh->func('reload', 'admin');
-
-
-=head1 EXPERIMENTAL SOFTWARE
-
-This package contains experimental software and should *not* be used
-in a production environment. We are following the Linux convention and
-treat the "even" releases (1.18xx as of this writing, perhaps 1.20xx,
-1.22xx, ... in the future) as stable. Only bug or portability fixes
-will go into these releases.
-
-The "odd" releases (1.19xx as of this writing, perhaps 1.21xx, 1.23xx
-in the future) will be used for testing new features or other serious
-code changes.
+    $rc = $drh->func( $database, '_CreateDB' );
+    $rc = $drh->func( $database, '_DropDB' );
 
 
 =head1 DESCRIPTION
 
 <DBD::mysql> and <DBD::mSQL> are the Perl5 Database Interface drivers for
 the mysql, mSQL 1.I<x> and mSQL 2.I<x> databases. The drivers are part
-of the I<Msql-Mysql-modules> package.
+of the I<mysql-modules> and I<Msql-modules> packages, respectively.
 
 
 =head2 Class Methods
@@ -354,19 +226,20 @@ of the I<Msql-Mysql-modules> package.
 
     use DBI;
 
-    $dsn = "DBI:mSQL:$database";
-    $dsn = "DBI:mSQL:database=$database;host=$hostname";
-    $dsn = "DBI:mSQL:database=$database;host=$hostname;port=$port";
-
-    $dbh = DBI->connect($dsn, undef, undef);
+    $dbh = DBI->connect("DBI:mSQL:$database", undef, undef);
+    $dbh = DBI->connect("DBI:mSQL:$database:$hostname", undef, undef);
+    $dbh = DBI->connect("DBI:mSQL:$database:$hostname:$port",
+			undef, undef);
 
         or
 
-    $dsn = "DBI:mysql:$database";
-    $dsn = "DBI:mysql:database=$database;host=$hostname";
-    $dsn = "DBI:mysql:database=$database;host=$hostname;port=$port";
+    use DBI;
 
-    $dbh = DBI->connect($dsn, $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database", $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname",
+			$user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
+			$user, $password);
 
 A C<database> must always be specified.
 
@@ -425,14 +298,7 @@ an empty list is returned.
 
 =item B<ListFields>
 
-Deprecated, see L</COMPATIBILITY ALERT> below. Used to be equivalent
-to
-
-    $sth = $dbh->prepare("LISTFIELDS $table");
-    $sth->execute;
-
-See L</SQL EXTENSIONS> below.
-
+Deprecated, see L</COMPATIBILITY ALERT> below.
 
 =item B<ListSelectedFields>
 
@@ -441,101 +307,33 @@ Deprecated, see L</COMPATIBILITY ALERT> below.
 =back
 
 
-=head2 Server Administration
+=head2 Database Manipulation
 
 =over 4
 
-=item admin
+=item B<CreateDB>
 
-    $rc = $drh->func("createdb", $dbname, [host, user, password,], 'admin');
-    $rc = $drh->func("dropdb", $dbname, [host, user, password,], 'admin');
-    $rc = $drh->func("shutdown", [host, user, password,], 'admin');
-    $rc = $drh->func("reload", [host, user, password,], 'admin');
-
-      or
-
-    $rc = $dbh->func("createdb", $dbname, 'admin');
-    $rc = $dbh->func("dropdb", $dbname, 'admin');
-    $rc = $dbh->func("shutdown", 'admin');
-    $rc = $dbh->func("reload", 'admin');
-
-For server administration you need a server connection. For obtaining
-this connection you have two options: Either use a driver handle (drh)
-and supply the appropriate arguments (host, defaults localhost, user,
-defaults to '' and password, defaults to ''). A driver handle can be
-obtained with
-
-    $drh = DBI->install_driver('mSQL');
-
-Otherwise reuse the existing connection of a database handle (dbh).
-
-There's only one function available for administrative purposes, comparable
-to the m(y)sqladmin programs. The command being execute depends on the
-first argument:
-
-=over 8
-
-=item createdb
-
-Creates the database $dbname. Equivalent to "m(y)sqladmin create $dbname".
-
-=item dropdb
-
-Drops the database $dbname. Equivalent to "m(y)sqladmin drop $dbname".
-
-It should be noted that database deletion is
-I<not prompted for> in any way.  Nor is it undo-able from DBI.
-
-    Once you issue the dropDB() method, the database will be gone!
-
-These method should be used at your own risk.
-
-=item shutdown
-
-Silently shuts down the database engine. (Without prompting!)
-Equivalent to "m(y)sqladmin shutdown".
-
-=item reload
-
-Reloads the servers configuration files and/or tables. This can be particularly
-important if you modify access privileges or create new users.
-
-=back
-
-
-=item B<_CreateDB>
-
-=item B<_DropDB>
-
-
-These methods are deprecated, see L</COMPATIBILITY ALERT> below.!
+=item B<DropDB>
 
     $rc = $drh->func( $database, '_CreateDB' );
     $rc = $drh->func( $database, '_DropDB' );
 
-      or
+These two methods allow programmers to create and drop databases from
+DBI scripts. Since mSQL disallows the creation and deletion of
+databases over the network, these methods explicitly connect to the
+mSQL daemon running on the machine C<localhost> and execute these
+operations there.
 
-    $rc = $drh->func( $host, $database, '_CreateDB' );
-    $rc = $drh->func( $host, $database, '_DropDB' );
+It should be noted that database deletion is I<not prompted for> in
+any way.  Nor is it undo-able from DBI.
 
-These methods are equivalent to the admin method with "createdb" or
-"dropdb" commands, respectively. In particular note the warnings
-concerning the missing prompt for dropping a database!
+    Once you issue the dropDB() method, the database will be gone!
+
+These methods should be used at your own risk.
 
 =back
 
 
-=head1 DATABASE HANDLES
-
-The DBD::mysql driver supports the following attributes of database
-handles (read only):
-
-    $infoString = $dbh->{'info'};
-    $threadId = $dbh->{'thread_id'};
-
-These correspond to mysql_info() and mysql_tread_id(), respectively.
-
-
 =head1 STATEMENT HANDLES
 
 The statement handles of DBD::mysql and DBD::mSQL support a number
@@ -544,25 +342,9 @@ of attributes. You access these by using, for example,
   my $numFields = $sth->{'NUM_OF_FIELDS'};
 
 Note, that most attributes are valid only after a successfull I<execute>.
-An C<undef> value will returned in that case. The most important exception
-is the C<mysql_use_result> attribute: This forces the driver to use
-mysql_use_result rather than mysql_store_result. The former is faster
-and less memory consuming, but tends to block other processes. (That's why
-mysql_store_result is the default.)
+An C<undef> value will returned in that case.
 
-To set the C<mysql_use_result> attribute, use either of the following:
-
-  my $sth = $dbh->prepare("QUERY", { "mysql_use_result" => 1});
-
-or
-
-  my $sth = $dbh->prepare("QUERY");
-  $sth->{"mysql_use_result"} = 1;
-
-Of course it doesn't make sense to set this attribute before calling the
-C<execute> method.
-
-Column dependent attributes, for example I<NAME>, the column names,
+Column dependant attributes, for example I<NAME>, the column names,
 are returned as a reference to an array. The array indices are
 corresponding to the indices of the arrays returned by I<fetchrow>
 and similar methods. For example the following code will print a
@@ -587,7 +369,7 @@ header of table names together with all rows:
       }
       print "\n";
   }
-
+x
 For portable applications you should restrict yourself to attributes with
 capitalized or mixed case names. Lower case attribute names are private
 to DBD::mSQL and DBD::mysql. The attribute list includes:
@@ -666,118 +448,98 @@ A reference to an array of table names, useful in a I<JOIN> result.
 
 A reference to an array of column types. It depends on the DBMS,
 which values are returned, even for identical types. mSQL will
-return types like &DBD::mSQL::INT_TYPE, &DBD::msql::TEXT_TYPE etc.,
-MySQL uses &DBD::mysql::FIELD_TYPE_SHORT, &DBD::mysql::FIELD_TYPE_STRING etc.
+return types like &mSQL::INT_TYPE, &msql::TEXT_TYPE etc., MySQL
+uses &mysql::FIELD_TYPE_SHORT, &mysql::FIELD_TYPE_STRING etc.
 
 =back
 
 
-=head1 SQL EXTENSIONS
-
-Certain metadata functions of mSQL and mysql that are available on the
-C API level, haven't been implemented here. Instead they are implemented
-as "SQL extensions" because they return in fact nothing else but the
-equivalent of a statement handle. These are:
-
-=over 4
-
-=item LISTFIELDS $table
-
-Returns a statement handle that describes the columns of $table.
-Ses the docs of msqlListFields or mysql_list_fields for details.
-
-=item LISTINDEX $table $index
-
-mSQL only; returns a statement handle that describes the index $index
-of table $table. See the docs of msqlListIndex for details.
-
-=back
-
 =head1 COMPATIBILITY ALERT
 
-Certain attributes methods have been declared obsolete or deprecated,
-partially because there names are agains DBI's naming conventions,
-partially because they are just superfluous or obsoleted by other methods.
+As of version 0.70 DBD::mSQL has a new maintainer. Even more, the sources
+have been completely rewritten in August 1997, so it seemed apropriate
+to bump the version number: Incompatibilities are more than likely.
 
-Obsoleted attributes and methods will be explicitly listed below. You cannot
-expect them to work in future versions, but they have not yet been scheduled
-for removal and currently they should be usable without any code modifications.
+=head2 Recent changes:
 
-Deprecated attributes and methods will currently issue a warning unless
-you set the variable $DBD::mSQL::QUIET to a true value. This will
-be the same for Msql-Mysql-modules 1.19xx and 1.20xx. They will be silently
-removed in 1.21xx.
+=over 2
 
-Here is a list of obsoleted attributes and/or methods:
+=item New connect method
 
-=over 4
+DBD::mSQL and DBD::mysql now use the new I<connect> method as introduced
+with DBI 0.83 or so. For compatibility reasons the old method still
+works, but the driver issues a warning when he detects use of the
+old version. There's no workaround, you must update your sources.
+(Sorry, but the change was in DBI, not in DBD::mysql and DBD::mSQL.)
 
-=item _CreateDB
+=item _ListFields returning statement handle
 
-=item _DropDB
+As of Msql-modules 1.1805, the private functions
 
-deprecated, use
+    $dbh->func($table, "_ListFields");
 
-    $drh->func("createdb", $dbname, $host, "admin")
-    $drh->func("dropdb", $dbname, $host, "admin")
+and
 
-=item _ListFields
+    $sth->func("_ListSelectedFields");
 
-deprecated, use
+no longer return a simple hash, but a statement handle.
+(I<_ListSelectedFields> is a stub now which just returns $self.)
+This should usually not be visible, when your statement handle gets
+out of scope. However, if your database handle (C<$dbh> in the
+above example) disconnects, either because you explicitly disconnect
+or because he gets out of scope, and the statement handle is still
+active, DBI will issue a warning for active cursors being destroyed.
 
-    $sth = $dbh->prepare("LISTFIELDS $table")
-    $sth->execute;
-
-=item _ListSelectedFields
-
-deprecated, just use the statement handles for accessing the same attributes.
-
-=item _NumRows
-
-deprecated, use
-
-    $numRows = $sth->rows;
-
-=item IS_PRI_KEY
-
-=item IS_NOT_NULL
-
-=item IS_KEY
-
-=item IS_BLOB
-
-=item IS_NUM
-
-=item LENGTH
-
-=item MAXLENGTH
-
-=item NUMROWS
-
-=item NUMFIELDS
-
-=item RESULT
-
-=item TABLE
-
-=item TYPE
-
-All these statement handle attributes are obsolete. They can be simply
-replaced by just downcasing the attribute names. You should expect them
-to be deprecated as of Msql-Mysql-modules 1.1821. (Whenever that will
-be.)
+The simple workaround is to execute C<$sth-E<gt>finish> or to ensure
+that C<$sth> gets out of scope before C<$dbh>. Sorry, but it was
+obvious nonsense to support two different things for accessing the
+basically same thing: A M(y)SQL result.
 
 =back
 
+The drivers do not conform to the current DBI specification in some minor
+points. For example, the private attributes I<is_num> or I<is_blob> have
+been written I<IS_NUM> and I<IS_BLOB>. For historical reasons we continue
+supporting the capitalized names, although the DBI specification now
+reserves capitalized names for standard names, mixed case for DBI and lower
+case for private attributes and methods.
+
+We currently consider anything not conforming to the DBI as deprecated.
+It is quite possible that we remove support of these deprecated names
+and methods in the future. In particular these includes:
+
+=over 4
+
+=item C<$sth-E<gt>func($table, '_ListSelectedFields')>
+
+highly deprecated, all attributes are directly accessible via the
+statement handle. For example instead of
+
+  $ref = $sth->func($table, '_ListSelectedFields')
+  my @names = $ref->{'NAME'}
+
+you just do a
+
+  my @names = @{$sth->{'NAME'}};
+
+=item Capitalized attribute names
+
+Deprecated, should be replaced by the respective lower case names.
+
+=back
 
 =head1 BUGS
 
 The I<port> part of the first argument to the connect call is
-implemented in an unsafe way when using mSQL. In fact it is just
-stting the environment variable MSQL_TCP_PORT during the connect
-call. If another connect call uses another port and the handles
-are used simultaneously, they will interfere. I doubt that this
-will ever change.
+implemented in an unsafe way. In fact it never did more than set
+the environment variable MSQL_TCP_PORT during the connect call. If
+another connect call uses another port and the handles are used
+simultaneously, they will interfere. In a future version this
+behaviour will hoefully change, depending on David and Monty. :-)
+
+The func method call on a driver handle seems to be undocumented in
+the DBI manpage. DBD::mSQL has func methods on driverhandles, database
+handles, and statement handles. What gives?
 
 Please speak up now (June 1997) if you encounter additional bugs. I'm
 still learning about the DBI API and can neither judge the quality of
@@ -865,35 +627,6 @@ such as books, magazine articles or CD-ROMs should be made to
 Alligator Descartes <I<descarte@hermetica.com>>.
 
 
-=head1 MAILING LIST SUPPORT
-
-This module is maintained and supported on a mailing list,
-
-    msql-mysql-modules@tcx.se
-
-To subscribe to this list, send a mail with the words
-
-    subscribe msql-mysql-modules
-
-or
-
-    subscribe msql-mysql-modules-digest
-
-in the first line of the body to mdomo@tcx.se. A mailing list archive is
-in preparation.
-
-Additionally you might try the dbi-user mailing list for questions about
-DBI and its modules in general. Subscribe via
-
-    http://www.fugue.com/dbi
-
-Mailing list archives are at
-
-     http://www.rosat.mpe-garching.mpg.de/mailing-lists/PerlDB-Interest/
-     http://outside.organic.com/mail-archives/dbi-users/
-     http://www.coe.missouri.edu/~faq/lists/dbi.html
-
-
 =head1 ADDITIONAL DBI INFORMATION
 
 Additional information on the DBI project can be found on the World
