@@ -2,70 +2,33 @@
 
 package Msql::Statement;
 
-@Msql::Statement::ISA = qw(DBI::st);
-
 use strict;
 use vars qw($OPTIMIZE $VERSION $AUTOLOAD);
 
-$VERSION = '1.19_11';
+$VERSION = '1.1817';
 
 $OPTIMIZE = 0; # controls, which optimization we default to
 
-sub fetchrow ($) {
-    my($self) = shift;
-    my($ref) = $self->fetchrow_arrayref;
-    if ($ref) {
-	@$ref;
-    } else {
-	();
-    }
-}
-sub fetchhash ($) {
-    my($self) = shift;
-    my($ref) = $self->fetchrow_hashref;
-    if ($ref) {
-	%$ref;
-    } else {
-	();
-    }
-}
-sub fetchcol ($$) {
-    my($self, $colNum) = @_;
-    my(@col);
-    $self->dataseek(0);
-    my($ref);
-    while ($ref = $self->fetchrow_arrayref) {
-	push(@col, $ref->[$colNum]);
-    }
-    @col;
-}
-sub dataseek ($$) {
-    my($self, $pos) = @_;
-    $self->func($pos, 'dataseek');
-}
-
-sub numrows { my($self) = shift; $self->rows() }
-sub numfields { my($self) = shift; $self->{'NUM_OF_FIELDS'} }
-sub affectedrows { my($self) = shift; $self->{'affected_rows'} }
-sub insertid { my($self) = shift; $self->{'insertid'} }
-sub arrAttr ($$) {
-    my($self, $attr) = @_;
-    my($arr) = $self->{$attr};
-    wantarray ? @$arr : $arr
-}
-sub table ($) { shift->arrAttr('table') }
-sub name ($) { shift->arrAttr('NAME') }
-sub type ($) { shift->arrAttr('type') }
-sub isnotnull ($) { shift->arrAttr('is_not_null') }
-sub isprikey ($) { shift->arrAttr('is_pri_key') }
-sub isnum ($) { shift->arrAttr('is_num') }
-sub isblob ($) { shift->arrAttr('is_blob') }
-sub length ($) { shift->arrAttr('length') }
+sub numrows    { my $x = shift; $x->{'NUMROWS'} or $x->fetchinternal( 'NUMROWS'
+  ) }
+sub numfields  { shift->fetchinternal( 'NUMFIELDS' ) }
+sub affectedrows { my $x = shift; $x->fetchinternal( 'AFFECTEDROWS') }
+sub insertid  { my $x = shift; $x->fetchinternal( 'INSERTID') }
+sub table      { return wantarray ? @{shift->fetchinternal('TABLE'    )}: shift->fetchinternal('TABLE'    )}
+sub name       { return wantarray ? @{shift->fetchinternal('NAME'     )}: shift->fetchinternal('NAME'     )}
+sub type       { return wantarray ? @{shift->fetchinternal('TYPE'     )}: shift->fetchinternal('TYPE'     )}
+sub isnotnull  { return wantarray ? @{shift->fetchinternal('ISNOTNULL')}: shift->fetchinternal('ISNOTNULL')}
+sub isprikey   { return wantarray ? @{shift->fetchinternal('ISPRIKEY' )}: shift->fetchinternal('ISPRIKEY' )}
+sub isnum     { return wantarray ? @{shift->fetchinternal('ISNUM' )}: shift->fetchinternal('ISNUM' )}
+sub isblob     { return wantarray ? @{shift->fetchinternal('ISBLOB' )}: shift->fetchinternal('ISBLOB' )}
+sub length     { return wantarray ? @{shift->fetchinternal('LENGTH'   )}: shift->fetchinternal('LENGTH'   )}
 
 sub maxlength  {
     my $sth = shift;
     my $result;
-    if (!($result = $sth->{'maxlength'})) {
+    if ($sth->isa("Mysql::Statement")) {
+	$result = $sth->fetchinternal('MAXLENGTH');
+    } else {
 	$result = [];
 	my ($l);
 	for (0..$sth->numfields-1) {
@@ -74,8 +37,8 @@ sub maxlength  {
 	$sth->dataseek(0);
 	my($col, @row, $i);
 	while (@row = $sth->fetchrow) {
-	    for ($i = 0;  $i < @row;  $i++) {
-		$col = $row[$i];
+	    $i = 0;
+	    while ($col = shift @row) {
 		my($s) = defined $col ? unctrl($col) : "NULL";
 		# New in 2.0: a string is longer than it should be
 		if (defined &Msql::TEXT_TYPE  &&
@@ -87,9 +50,9 @@ sub maxlength  {
 		if (length($s) > $$result[$i]) {
 		    $$result[$i] = length($s);
 		}
+		++$i;
 	    }
 	}
-	$sth->{MAXLENGTH} = $result;
     }
     return wantarray ? @$result : $result;
 }
@@ -97,7 +60,7 @@ sub maxlength  {
 sub listindices {
     my($sth) = shift;
     my(@result,$i);
-    if (!&Msql::IDX_TYPE()) {
+    if ($sth->isa('Mysql::Statement')  ||  !&Msql::IDX_TYPE) {
 	return ();
     }
     foreach $i (0..$sth->numfields-1) {
@@ -147,7 +110,8 @@ sub as_string {
     }
     for (0..$sth->numfields-1) {
 	$l=length($sth->name->[$_]);
-	if ($sth->optimize  &&  $l < $sth->maxlength->[$_]) {
+	if (($sth->optimize  ||  $sth->isa("Mysql::Statement"))  &&
+	    $l < $sth->maxlength->[$_]) {
 	    $l= $sth->maxlength->[$_];
 	}
 	if (!$sth->isnotnull  &&  $l < 4) {
@@ -163,18 +127,18 @@ sub as_string {
     $sth->dataseek(0);
     my(@row);
     while (@row = $sth->fetchrow) {
-	my ($col, $pcol, @prow, $i, $j);
-	for ($i = 0;  $i < $sth->numfields;  $i++) {
-	    $col = $row[$i];
-	    $j = @prow;
+	my ($col, $pcol, @prow);
+	while ($col = shift @row) {
 	    $pcol = defined $col ? unctrl($col) : "NULL";
 	    # New in 2.0: a string is longer than it should be
-	    if (defined &Msql::TEXT_TYPE  &&
+	    if (!$sth->isa('Mysql::Statement')  &&
+		defined &Msql::TEXT_TYPE  &&
 		$sth->optimize &&
-		$sth->type->[$j] == &Msql::TEXT_TYPE &&
-		length($pcol) > $sth->length->[$j] + 5) {
+		$sth->type->[$_] == &Msql::TEXT_TYPE &&
+		length($pcol) > $sth->length->[$_] + 5
+		) {
 		my $l = length($col);
-		substr($pcol,$sth->length->[$j])="...($l)";
+		substr($pcol,$sth->length->[$_])="...($l)";
 	    }
 	    push(@prow, $pcol);
 	}
@@ -187,3 +151,15 @@ sub as_string {
 }
 
 1;
+
+sub _leftjustify($$) {
+    my ($self, $type) = @_;
+    if ($self->isa('Mysql::Statement')) {
+	($type == Mysql::FIELD_TYPE_CHAR())
+	|| ($type == Mysql::FIELD_TYPE_STRING())
+        || ($type == Mysql::FIELD_TYPE_VAR_STRING());
+    } else {
+	$type & (&Msql::CHAR_TYPE |
+		 (defined &Msql::TEXT_TYPE ? Msql::TEXT_TYPE() : 0));
+    }
+}
