@@ -1,22 +1,22 @@
 # -*- perl -*-
 
 package Mysql;
+BEGIN { require 5.002 }
+# use vars qw($db_errstr); # 5.003
+$db_errstr = $db_errstr = '';
 
-BEGIN { require 5.004 };
-
-require Carp;
-require DynaLoader;
-require Exporter;
-require DBI;
 require Mysql::Statement;
-require DBD::mysql;
+$QUIET  = $QUIET  = '';
+@ISA    = @ISA    = '';
+@EXPORT = @EXPORT = '';
+$VERSION = '1.1817';
 
-use vars qw($QUIET @ISA @EXPORT @EXPORT_OK $VERSION $db_errstr);
+$QUIET = 0;
 
-$db_errstr = '';
-$QUIET  = 0;
-@ISA    = qw(DBI); # Inherits Exporter and DynaLoader via DBI
-$VERSION = '1.19_11';
+use Carp ();
+use DynaLoader ();
+use Exporter ();
+@ISA = ('Exporter', 'DynaLoader');
 
 # @EXPORT is a relict from old times...
 @EXPORT = qw(
@@ -36,159 +36,24 @@ $VERSION = '1.19_11';
 		SYSVAR_TYPE
 	       );
 
-my($FETCH_map) = {
-    'HOST' => '_host',
-    'DATABASE' => 'database'
-};
+sub host     { return shift->{'HOST'} }
+sub sock     { return shift->{'SOCK'} }
+sub sockfd   { return shift->{'SOCKFD'} }
+sub database { return shift->{'DATABASE'} }
 
-sub FETCH ($$) {
-    my($self, $key) = @_;
-    if ($key eq 'COMPATIBILITY') {
-	return $self->{'COMPATIBILITY'};
-    }
-    if (exists($FETCH_map->{$key})) {
-	$key = $FETCH_map->{$key};
-    }
-    my($dbh) = $self->{'dbh'};
-    my($attr) = $dbh->{'private_DBD_mysql_attr'};
-    if (exists($attr->{$key})) {
-	$attr->{$key};
-    } else {
-	$dbh->{$key};
+sub quote	{
+    my $self = shift;
+    my $str = shift;
+    return 'NULL' unless defined $str;
+    my $trunc = shift;
+    substr($str,$trunc) = '' if defined $trunc and $trunc > 0 and length($str) > $trunc;
+    $str =~ s/([\\\'])/\\$1/g;
+    if ($self->isa('Mysql')) {
+	$str =~ s/\0/\\0/g;
     }
+    "'$str'";
 }
 
-sub STORE ($$$) {
-    my($self, $key, $val) = @_;
-    if ($key eq 'COMPATIBILITY') {
-	$self->{'COMPATIBILITY'} = $val;
-    } else {
-	$self->{'dbh'}->{$key} = $val;
-    }
-}
-
-sub connect ($;$$$$) {
-    my($class, $host, $db, $user, $password) = @_;
-    my($self) = { 'host' => ($host || ''),
-		  'user' => $user,
-		  'password' => $password,
-		  'db' => $db,
-	          'driver' => 'mysql',
-	          'COMPATIBILITY' => 1 };
-    bless($self, $class);
-    $self->{'drh'} = DBI->install_driver($self->{'driver'});
-    if ($db) {
-	my($dsn) = "DBI:mysql:database=$db;host=$host";
-	if (!($self->{'dbh'} = $class->SUPER::connect($dsn, $user, $password))) {
-	    $db_errstr = $DBI::errstr;
-	    return undef;
-	}
-	$dbh->{'CompatMode'} = 1;
-	$dbh->{'PrintError'} = !$Mysql::QUIET;
-    }
-    $self;
-}
-
-sub selectdb ($$) {
-    my($self, $db) = @_;
-    my($dsn) = "DBI:mysql:database=$db:host=" . $self->{'host'};
-    my($dbh) = DBI->connect($dsn, $self->{'user'}, $self->{'password'});
-    if (!$dbh) {
-	$db_errstr = $self->{'errstr'} = $DBI::errstr;
-	$self->{'errno'} = $DBI::err;
-	undef;
-    } else {
-	$self->{'dbh'} = $dbh;
-	$self->{'db'} = $db;
-	$self;
-    }
-}
-
-sub listdbs ($) {
-    my($self) = shift;
-    $self->{'drh'}->func($self->{'host'}, '_ListDBs');
-}
-
-sub listtables ($) {
-    my($self) = shift;
-    $self->{'dbh'}->func('_ListTables');
-}
-
-sub quote ($$) { my($self) = shift; $self->{'dbh'}->quote(@_); }
-
-sub errmsg ($) {
-    my($self) = shift;
-    if (!ref($self)) {
-	$db_errstr;
-    } elsif ($self->{'dbh'}) {
-	$self->{'dbh'}->errstr();
-    } else {
-	$self->{'drh'}->errstr();
-    }
-}
-
-sub errno ($) {
-    if ($self->{'dbh'}) {
-	$self->{'dbh'}->err();
-    } else {
-	$self->{'drh'}->err();
-    }
-}
-
-sub listfields ($$) {
-    my($self, $table) = @_;
-    $self->query("LISTFIELDS $table");
-}
-
-sub query ($$) {
-    my($self, $statement) = @_;
-    my($sth) = $self->{'dbh'}->prepare($statement);
-    my($result);
-    $sth->{'PrintError'} = !$Mysql::QUIET;
-    if ($sth  &&  !($result = $sth->execute())) {
-	undef $sth;
-    }
-    $sth->{'CompatMode'} = 1;
-    $sth;
-}
-
-sub shutdown ($) {
-    my($self) = shift;
-    if ($self->{'dbh'}) {
-	$self->{'dbh'}->admin('shutdown', 'admin');
-    } else {
-	$self->{'drh'}->func('shutdown', $self->{'host'}, $self->{'user'},
-			     $self->{'password'}, 'admin');
-    }
-}
-
-sub createdb ($$) {
-    my($self, $db) = @_;
-    if ($self->{'dbh'}) {
-	$self->{'dbh'}->admin('createdb', $db, 'admin');
-    } else {
-	$self->{'drh'}->func('createdb', $db, $self->{'host'},
-			     $self->{'user'}, $self->{'password'}, 'admin');
-    }
-}
-
-sub dropdb ($$) {
-    my($self, $db) = @_;
-    if ($self->{'dbh'}) {
-	$self->{'dbh'}->admin('dropdb', $db, 'admin');
-    } else {
-	$self->{'drh'}->func('dropdb', $db, $self->{'host'},
-			     $self->{'user'}, $self->{'password'}, 'admin');
-    }
-}
-
-sub host     ($) { shift->{'host'} }
-sub database ($) { shift->{'db'} }
-sub info ($) { shift->{'dbh'}->{'info'} }
-sub sock ($) { shift->{'dbh'}->{'sock'} }
-sub sockfd ($) { shift->{'dbh'}->{'sockfd'} }
-
-
 sub AUTOLOAD {
     my $meth = $AUTOLOAD;
     my $converted = 0;
@@ -202,7 +67,7 @@ sub AUTOLOAD {
 
 
     TRY: {
-	my $val = DBD::mysql::constant($meth, @_ ? $_[0] : 0);
+	my $val = constant($meth, @_ ? $_[0] : 0);
 	if ($! == 0) {
 	    eval "sub $AUTOLOAD { $val }";
 	    return $val;
@@ -230,23 +95,7 @@ sub AUTOLOAD {
       . " autoloadable (last try $meth)";
 }
 
-
-sub gethostinfo ($) { shift->{'dbh'}->{'hostinfo'} }
-sub getprotoinfo ($) { shift->{'dbh'}->{'protoinfo'} }
-sub getserverinfo ($) { shift->{'dbh'}->{'serverinfo'} }
-sub getserverstats ($) { shift->{'dbh'}->{'stats'} }
-
-
-Mysql->init_rootclass();
-
-package Mysql::dr;
-@Mysql::dr::ISA = qw(DBI::dr);
-
-package Mysql::db;
-@Mysql::db::ISA = qw(DBI::db);
-
-package Mysql::st;
-@Mysql::st::ISA = qw(Mysql::Statement);
+bootstrap Mysql $VERSION;
 
 1;
 __END__
@@ -269,7 +118,7 @@ Msql / Mysql - Perl interfaces to the mSQL and mysql databases
 
   $dbh = Mysql->connect(undef, $database, $user, $password);
   $dbh = Mysql->connect($host, $database, $user, $password);
-
+	
   $dbh->selectdb($database);
 	
   @arr = $dbh->listdbs;
@@ -298,13 +147,6 @@ Msql / Mysql - Perl interfaces to the mSQL and mysql databases
   $rc = $dbh->createdb($database);
   $rc = $dbh->dropdb($database);
 
-=head1 OBSOLETE SOFTWARE
-
-As of Msql-Mysql-modules 1.19_10 M(y)sqlPerl is no longer a separate module.
-Instead it is emulated using the DBI drivers. You are strongly encouraged
-to implement new code with DBI directly. See L<COMPATIBILITY NOTES>
-below.
-
 =head1 DESCRIPTION
 
 This package is designed as close as possible to its C API
@@ -312,11 +154,8 @@ counterpart. The manual that comes with mSQL or MySQL describes most things
 you need. Due to popular demand it was decided though, that this interface
 does not use StudlyCaps (see below).
 
-As of March 1998, the Msql and Mysql modules are obsoleted by the
-DBI drivers DBD::mSQL and DBD::mysql, respectively. You are strongly
-encouraged to implement new code with the DBI drivers. In fact,
-Msql and Mysql are currently implemented as emulations on top of
-the DBI drivers.
+The version you have selected is an adaption still under development,
+please consult the file "Changes" in your distribution.
 
 Internally you are dealing with the two classes C<Msql> and
 C<Msql::Statement> or C<Mysql> and C<Mysql::Statement>, respectively.
@@ -444,11 +283,9 @@ If you want to know in greater detail what's going on, set the
 environment variables that are described in David's manual. David's
 debugging aid is excellent, there's nothing to be added.
 
-By default errors are printed as warnings. You can suppress this
-behaviour by using the PrintError attribute of the respective handles:
-
-    $dbh->{'dbh'}->{'PrintError'} = 0;
-
+If you want to use the C<-w> switch but do not want to see the error
+messages from the msql daemon, you can set the variables $Msql::QUIET
+or $Mysql::QUIET to some true value, and they will be supressed.
 
 =head2 -E<gt>quote($str [, $length])
 
@@ -723,62 +560,9 @@ MySQL is a libmysqlclient.a library written by Michael Widenius
 This was originally inspired by MySQL.
 
 
-=head1 COMPATIBILITY NOTES
-
-M(y)sql used to be a separate module written in C. This is no longer
-the case, instead the old modules are emulated by their corresponding
-DBI drivers. I did my best to remove any incompatibilities, but the
-following problems are known to remain:
-
-=over 4
-
-=item Static methods
-
-For whatever reason, mSQL implements some functions independent from
-the respective database connection that really depend on it. This
-made it possible to implement
-
-    Msql->errmsg
-
-or
-
-    Msql->getserverinfo
-
-as static methods. This is no longer the case, it never was for
-MysqlPerl. Instead you have to use
-
-    $dbh->errmsg
-
-or
-
-    $dbh->getserverinfo
-
-=item $M(Y)SQL::QUIET
-
-This variable used to turn off the printing of error messages. Unfortunately
-DBI uses a completely different mechanism for that: The C<PrintError>
-attribute of the database and/or statement handles. We try to emulate
-the old behaviour by setting the C<PrintError> attribute to the current
-value of $M(Y)SQL::QUIET when a handle is created, that is when
-M(y)sql->connect or $dbh->query() are called.
-
-You can overwrite this by using something like
-
-    $dbh->{'dbh'}->{'PrintError'} = 1;
-
-or
-
-    $sth->{'PrintError'} = 0;
-
-=back
-
-
-=head1 AUTHORS
-
-Andreas Koenig C<koenig@franz.ww.TU-Berlin.DE> wrote the original
-MsqlPerl. Jochen Wiedmann C<joe@ispsoft.de> wrote the M(y)sqlPerl
-emulation using DBI.
+=head1 AUTHOR
 
+andreas koenig C<koenig@franz.ww.TU-Berlin.DE>
 
 =head1 SEE ALSO
 
